/********************************************************/
/*                                                      */
/*               OO7 Benchmark                          */
/*                                                      */
/*              COPYRIGHT (C) 1993                      */
/*                                                      */
/*                Michael J. Carey 		        */
/*                David J. DeWitt 		        */
/*                Jeffrey Naughton 		        */
/*               Madison, WI U.S.A.                     */
/*                                                      */
/*	         ALL RIGHTS RESERVED                    */
/*                                                      */
/********************************************************/

#include <libc.h>
#include <malloc.h>
#include <stdio.h>
#include "oo7.h"
#include "globals.h"
#include "GenParams.h"
#include "VarParams.h"

////////////////////////////////////////////////////////////////////////////
//
// Document Methods
//
////////////////////////////////////////////////////////////////////////////

extern int	debugMode;

////////////////////////////////////////////////////////////////////////////
//
// Document Constructor
//
////////////////////////////////////////////////////////////////////////////

Document::Document(dbint cpId, CompositePart* cp)
{
    dbchar 	*p;
    char	myText[DocTextLength];

    // add document to extent of all documents
    // dts 3-11-93 AllDocuments is never used, so don't maintain it.
    // AllDocuments.add(this);

    // fill in id
    id = cpId; 
    // fill in back pointer to composite part
    part = cp;

    // prepare and fill in the document title
    char docTitle[TitleSize];
    sprintf(docTitle, "Composite Part %08d", cpId);
    if (debugMode) {
        printf("Document::Document(title = %s)\n", docTitle);
    }
    strncpy(title, docTitle, TitleSize);

    // prepare and fill in the document text
    sprintf(myText, DocumentText, cpId);
    int stringLen = strlen(myText);

    // allocate the space to hold the actual document
    p = (dbchar *) new (DocumentSet) dbchar[(DocumentSize)];

    // initialize the document
    int curChar, strChar;
    curChar = strChar = 0;
    while (curChar < DocumentSize-1) {
        p[curChar++] = myText[strChar++];
        if (strChar >= stringLen) { strChar = 0; }
    }
    p[curChar] = '\0';

    // now make the document object point at the actual document text
    text = p; 
}

// destructor
Document::~Document()
{
    delete text;  // delete actual document text
    // remove document from extent of all documents
    // dts 3-11-93 AllDocuments is never used, so don't maintain it.
    // AllDocuments.remove (this);
}

////////////////////////////////////////////////////////////////////////////
//
// Document searchText Method for use in traversals
//
////////////////////////////////////////////////////////////////////////////

int Document::searchText(char c)
{
    dbchar x;
	
    if (debugMode) {
        printf("                    Document::searchText(title = %s)\n", title);
    }
    x = c;

    // count occurrences of the indicated letter (for busy work)
    int i = 0; 
    int count = 0;
#ifdef SEARCH_BY_CHAR
    // character by character search version
    // do some manual cse elimination for E
    {
	dbchar *text = this->text;
    	while (text[i] != '\0') {
	    if (text[i++] == x) { count++; }
    	}
    }
#else // use a db strings function for better efficiency
    // use index function to repeatedly look for the character
    // until the string is exhausted, counting each found occurance
    {
	dbchar * indexpt = this->text;
	while( indexpt = index(indexpt,x))
	{
		count++;
		// skip to next character
		indexpt++;
	}
    }
#endif

    if (debugMode) {
        printf("                    [found %d '%c's among %d characters]\n",
	        		     count, c, i);
    }
    return count;
}


////////////////////////////////////////////////////////////////////////////
//
// Document replaceText Method for use in traversals
//
////////////////////////////////////////////////////////////////////////////

int Document::replaceText(char *oldString, char *newString)
{
    dbchar *newText;
    int   newSize;

    if (debugMode) {
        printf("                    Document::changeText(title = %s)\n", title);
    }

    // check to see if the text starts with the old string

    int oldTextLength = strlen(text) + 1;
    int oldStrLength  = strlen(oldString);
    int foundMatch = (strncmp(text, oldString, oldStrLength) == 0);

    // if so, change it to start with the new string instead
    if (foundMatch) 
    {
        int newStrLength  = strlen(newString);
        int lengthDiff = newStrLength - oldStrLength;
        int newTextLength  = oldTextLength + lengthDiff;

	if (lengthDiff == 0) {
            strncpy(text, newString, newStrLength);
        } else {
	// delete/insert method: delete the old string and insert new
	// one, using bins/bdel
	    bdel(text,oldStrLength);
	    bins(text,newString,newStrLength);
	}
    }

    if (debugMode) 
    {
	if (foundMatch) {
            printf("                    [changed \"%s\" to \"%s\"]\n",
	                                 oldString, newString);
	} else {
            printf("                    [no match, so no change was made]\n");
	}
    }

    if (foundMatch) {
        return 1;

    } else {
        return 0;
    }
}


