/********************************************************/
/*                                                      */
/*               OO7 Benchmark                          */
/*                                                      */
/*              COPYRIGHT (C) 1993                      */
/*                                                      */
/*                Michael J. Carey 		        */
/*                David J. DeWitt 		        */
/*                Jeffrey Naughton 		        */
/*               Madison, WI U.S.A.                     */
/*                                                      */
/*	         ALL RIGHTS RESERVED                    */
/*                                                      */
/********************************************************/


#include <libc.h>
#include <string.h>
#include <stdio.h>

#include "oo7.h"
#include "GenParams.h"
#include "VarParams.h"
#include "IntBtree.h"
#include "globals.h"

/////////////////////////////////////////////////////////////////
//
// Traversal #7.  Pick a random atomic part; traverse from the 
// part to its containing composite part P.  From P, traverse
// backwards up to all base assemblies referencing P; for each
// such base assembly, traverse upward through the assembly
// hierarchy until the root is reached.  This traversal coupled
// with traversal #1 ensures
// that all pointers are bi-directional.
//
/////////////////////////////////////////////////////////////////

extern int	debugMode;

//////////////////////////////////////////////////////////////////
//
// Routine to initiate Traversal #7
//
//////////////////////////////////////////////////////////////////


int traverse7()
{
    int 	partId;
    AtomicPart* atomH;

    // get access to the design library containing atomic parts

    // prepare to select a random atomic part via the id index
    // now randomly select a part and traverse upwards from there

    // set random seed so "hot" runs are truly hot
    srandom(1);

    partId = (int) (random() % TotalAtomicParts) + 1;
    atomH = (AtomicPart*) AtomicPartIdx->lookup(partId);
    if (atomH == NULL) {
        fprintf(stderr, "ERROR: Unable to find atomic part %d\n", partId);
	E_AbortTransaction();
        exit(1);
    }

    if (debugMode) {
        printf("  In Traverse7, starting at atomic part %d\n", partId);
    }
    return atomH->partOf->traverse7();
}



//////////////////////////////////////////////////////////////////
//
// CompositePart Method for Traversal #7
//
//////////////////////////////////////////////////////////////////


int CompositePart::traverse7()
{
    AssocItr* baseI;  // iterator for traversing baseassemblies parts
    BaseAssembly* ba; 
    PartIdSet visitedBaseIds;
    PartIdSet visitedComplexIds;

    if (debugMode) {
        printf("\tCompositePart::traverse7(id = %d)\n", id);
    }

    // search up the design hierarchy (along the private path)
    int count = 0;

    // establish iterator of private base assemblies
    baseI = new AssocItr(usedInPriv); 
    ba = (BaseAssembly*) baseI->next(); 
    while (ba != NULL)
    {
        if (!visitedBaseIds.contains(ba->id)) {
            visitedBaseIds.insert(ba->id);
	    count += ba->traverse7(visitedComplexIds);
	}
        ba = (BaseAssembly*) baseI->next(); 
    }
    delete baseI;
    return count;
}


//////////////////////////////////////////////////////////////////
//
// BaseAssembly Method for Traversal #7
//
//////////////////////////////////////////////////////////////////


int BaseAssembly::traverse7(PartIdSet& visitedComplexIds)
{
    if (debugMode) {
        printf("\t\tBaseAssembly::traverse7(id = %d)\n", id);
    }

    // visit this assembly and move on up the design hierarchy
    int count = 1;

    DoNothing();
    if (!visitedComplexIds.contains(superAssembly->id)) {
        count += superAssembly->traverse7(visitedComplexIds);
    }
    return count;
}


//////////////////////////////////////////////////////////////////
//
// ComplexAssembly Method for Traversal #7
//
//////////////////////////////////////////////////////////////////


int ComplexAssembly::traverse7(PartIdSet& visitedComplexIds)
{
    if (debugMode) {
        printf("\t\t\tComplexAssembly::traverse7(id = %d)\n", id);
    }

    // process this assembly and move on up the design hierarchy
    int count = 1;
    visitedComplexIds.insert(id);
    DoNothing();
    if (superAssembly != NULL) {
	if (!visitedComplexIds.contains(superAssembly->id)) {
            count += superAssembly->traverse7(visitedComplexIds);
        }
    }
    return count;
}

